
function [Obj] = animation(t, Obj, f, varargin)

%ANIMATION performs animation of the Robot Toolbox's objects
% 
% syntax:
% [Obj] = animation(t, Obj, f, ...)
% [Obj] = animation(t, Obj, f, ..., delay)
% [Obj] = animation(t, Obj, f, ..., delay, trace)
% [Obj] = animation(t, Obj, f, ..., delay, trace, args)
%
% input:
% Obj       - handle to Robot Toolbox object to be animated
% f         - function determining configuration of Obj for a given time instant
% Obj,f,... - pairs: handle and function for each animated object
% delay     - pause between animation steps (in seconds), default 0
% trace     - determines method of motion's plotting 
%             0 - in each time instant only current motion's phase is visible
%             1 - all motion's phases are visible, 
%             default 0
% args      - list of user arguments passed to each of the function "f"
%
% Function "f" should determine cofiguration of the object (transformation
% matrix for block and csys and configuration vector for robot) for a given
% time instant. This function must be of the form:
%
%   function [conf] = myfun(Obj, t, flag, args)
%
% "Obj" is an object which configuration has to be determined; "t" is a
% given time; "flag" is equal to 'init' for first call of the function "f"
% (output argument is not taken into account in this case) and "flag" is
% empty otherwise; "args" additional arguments passed by function 
% animation if user determined them. "conf" is a configuration of the "Obj" 
% for the time "t".
%
% output:
% Obj - handle's vector of an animated objects
%
% example:
%
% axis([-5 5 -5 5 -1 9]);
% view([45, 40])
% camlight
% r1 = robot([dh(0,0,0,0)], ['r'], 'RobotCylinderRadius', 0.5, ...
%     'RobotGrasper', [3;0;0], 'RobotGrasperSize', [1,2]);
% r2 = robot([dh(0,0,0,0)], ['r'], 'RobotCylinderRadius', 0.5, 'RobotBase', [0;0;2], ...
%     'RobotGrasper', [3;0;0], 'RobotGrasperSize', [1,2]);
% robot_plot(r1, 'surf');
% robot_plot(r2, 'surf');
% b = block_move(cube(1), [0;-5;4]);
% block_plot(b)
% cs = csys([-4;4;3]);
% csys_plot(cs)
% f1 = inline('t*pi/10', 'R', 't', 'flag');
% f2 = inline('t*pi/20', 'R', 't', 'flag');
% f3 = inline('[0;t-5;4]', 'B', 't', 'flag');
% f4 = inline('rp2t(rotz(t*pi/10),[-4;4;3])', 'CS', 't', 'flag');
% animation(0:0.1:10, r1, f1, r2, f2, b, f3, cs, f4, 0.001)
% shows animation of two robots, one cube and one coordinate system;
% the animation's time is equal to 10s, the first robot is moved from
% configuration 0 to pi radians, the second robot is moved from 0 to pi/2 
% radians, the cube is moved from position [0;-5;4] to [0;5;4], the
% coordinate system is rotated around z-axis by pi radians without changing
% the position
%
% See also: BLOCK, BLOCK_MOTION, CSYS, ROBOT, ROBOT_MOTION
%

global RTHND;

if ( ~isnumeric(t)          ) rethrow(geterr('argValVect0', 't')); end
if ( ~parIsObj(Obj)         ) rethrow(geterr('argValObj2', 'Obj')); end
if ( RTHND{Obj}.owner ~= 0  ) rethrow(geterr('argValObj0','Obj','biblioteki RobotToolbox')); end
if ( ischar(f) ) f=str2func(f); end
if ( ~isa(f,'function_handle') & ~isa(f,'inline')) rethrow(geterr('argValFun', 'f')); end

Obj = [Obj];  fun = {f};  tp = {gettype(Obj)};
for i=1:2:length(varargin)
  obj = varargin{i};
  if ( parIsObj(obj) ) 
    if ( RTHND{obj}.owner ~= 0  ) rethrow(geterr('argValObj0','Obj','biblioteki RobotToolbox')); end
    if ( i < length(varargin) )
      f = varargin{i+1};
      if ( ischar(f) ) f=str2func(f); end
      if ( ~isa(f,'function_handle') & ~isa(f,'inline')) rethrow(geterr('argValFun', 'f')); end
      Obj = [Obj obj];  fun = {fun{:} f};  tp = { tp{:}, gettype(obj)};
    else rethrow(geterr('argValFun', 'f')); end
  else break; end
end

varargin = varargin(2*(length(Obj)-1)+1:length(varargin));

len = length(varargin);
trace = 0;  delay = 0.0;
if ( len > 1 )
  delay = varargin{1};  trace = varargin{2};
  varargin = varargin(3:len);
elseif ( len > 0 )
  delay = varargin{1};  
  varargin = varargin(2:len);
end
if ( ~parIsVector(delay,1) ) rethrow(geterr('argValNum', 'delay')); end
if ( ~parIsVector(trace,1) ) rethrow(geterr('argValNum', 'trace')); end

if ( length(varargin) > 0 ) args=varargin;
else args={}; end

c = {};
for i=1:length(fun)
  try fun{i}(Obj(i),t(1),'init',args{:}); 
  catch rethrow(geterr('argValFun1', i)); end
  c{i} = [];
end

obj = [];  axs = [];
for i=1:length(Obj)
  if ( RTHND{Obj(i)}.fig.axes ~= -1 ) 
    obj = [obj i]; 
    axs = [axs, getowner(Obj(i), 'axes')];
  end
end;

err = 0;
if ( ~isempty(obj) )
  for i=obj hold(RTHND{Obj(i)}.fig.axes, 'on'); end
  for tm=t
    for i=obj
      conf = fun{i}(Obj(i), tm, '', args{:});
      if ( isempty(conf) | isequal(conf,c{i}) ) continue; end
      switch ( tp{i} )
        case 'block'
          if ( parIsMatrix(conf,3,1)      ) conf=rp2t(eye(3),conf); 
          elseif ( parIsMatrix(conf,3,3)  ) conf=rp2t(conf,[0;0;0]); 
          elseif ( ~parIsMatrix(conf,4,4) ) err=i; break; end          
          blkmv(Obj(i), conf, 0);
          blkrdrw(Obj(i), ~trace);
        case 'csys'
          if ( parIsMatrix(conf,3,1)      ) conf=rp2t(eye(3),conf); 
          elseif ( parIsMatrix(conf,3,3)  ) conf=rp2t(conf,[0;0;0]); 
          elseif ( ~parIsMatrix(conf,4,4) ) err=i; break; end
          csmv(Obj(i), conf, 0);
          csrdrw(Obj(i), ~trace);
        case 'robot'
          if ( ~isnumeric(conf) ) err=i; break; end
          rcfgset(Obj(i), conf);
          rrdrw(Obj(i), axs(i), ~trace);
      end
      c{i} = conf;
    end
    if ( err ) break; end
    drawnow; pause(delay)
  end
  for i=obj hold(RTHND{Obj(i)}.fig.axes, 'off'); end
end

if ( err ) switch ( tp{err} )
  case 'block', rethrow(geterr('retValT', Obj(err)));
  case 'csys',  rethrow(geterr('retValT', Obj(err)));
  case 'robot', rethrow(geterr('retValQ', Obj(err)));
end; end

for i=setdiff(1:length(Obj), obj)
  conf = fun{i}(Obj(i), t(length(t)), '', args{:});
  if ( isempty(conf) ) continue; end
  switch ( tp{i} )
    case 'block'
      if ( parIsMatrix(conf,3,1)      ) conf=rp2t(eye(3),conf); 
      elseif ( parIsMatrix(conf,3,3)  ) conf=rp2t(conf,[0;0;0]); 
      elseif ( ~parIsMatrix(conf,4,4) ) err=i; break; end
      blkmv(Obj(i), conf, 0);
    case 'csys'
      if ( parIsMatrix(conf,3,1)      ) conf=rp2t(eye(3),conf); 
      elseif ( parIsMatrix(conf,3,3)  ) conf=rp2t(conf,[0;0;0]); 
      elseif ( ~parIsMatrix(conf,4,4) ) err=i; break; end
      csmv(Obj(i), conf, 0);
    case 'robot'
      if ( ~isnumeric(conf) ) err=i; break; end
      rcfgset(Obj(i), conf);
  end
end;

if ( err ) switch ( tp{err} )
  case 'block', rethrow(geterr('retValT', Obj(err)));
  case 'csys',  rethrow(geterr('retValT', Obj(err)));
  case 'robot', rethrow(geterr('retValQ', Obj(err)));
end; end

if ( nargout == 0 ) clear Obj; end
