
function [B] = block_complex(varargin)

%BLOCK_COMPLEX creates one complex block from given blocks or
%              connects given blocks to existing complex block
% 
% syntax:
% [B] = block_complex(B, ...)
%
% A new block is an union of few blocks, each component is a separate 
% block, it has its own handle and properties. Created blok has not own
% properties, its appearance is defined by properties of its components.
%
% input:
% B,... - a list of handles of blocks to be connected
%
% output:
% B - handle to a new complex block or first block in the input argument
%     list
%
% example:
%
% b1 = cuboid(2,1,1,'BlockColor','b');
% b2 = block_move(pyramid(2,1,1,'BlockColor','r'),[0;0;1]);
% b  = block_complex(b1, b2);
% block_plot(b)
% campos([-4,-3,3])
% creates and plots a new complex block composed of cuboid and pyramid on its top
%
% b1 = block_complex(cuboid(2,1,1), block_move(pyramid(2,1,1),[0;0;1]));
% b2 = block_move(sphere(0.25),[0;0;2.25]);
% block_complex(b1, b2);
% block_plot(b1)
% campos([-4,-3,3])
% creates a new complex block composed of cuboid and pyramid on its top (b1)
% next creates a sphere (b2) and connects it to existing complex block
%
% See also: BLOCK, BLOCK_MERGE
%

global RTHND;

if ( length(varargin) > 0 )
    
  for i=1:length(varargin)
    blk = varargin{i};
    if ( ~parIsObj(blk,'block') ) rethrow(geterr('argValObj','B','block')); end
    if (( RTHND{blk}.owner ~= 0 ) & parIsObj(RTHND{blk}.owner,'block') ) 
      rethrow(geterr('argValObj0','B','block')); 
    end
  end

  B = varargin{1};
  varargin = varargin(2:length(varargin));
  if ( ~parIsObj(B,'block:complex') )
    blk = B;
    B   = gethnd;
    b   = blkcmplxcreate(B,blk,varargin{:});
    RTHND{B} = b;
  elseif ( length(varargin) > 0 )
    RTHND{B} = blkcmplxattach(RTHND{B},varargin{:});
  end

  if ( RTHND{B}.fig.axes ~= -1 )
    fhndadd(B);
    hold(RTHND{B}.fig.axes, 'on');
    B = blkrdrw(B,1);
    hold(RTHND{B}.fig.axes, 'off');
  end
  
else 

  B = gethnd;
  b = blkcmplxcreate(B);
  RTHND{B} = b;
  
end
    
if ( nargout == 0 ) clear B; end
