
function [val] = block_get(B, varargin)

%BLOCK_GET returns the value of the block's property
% 
% syntax:
% [val] = block_get(B, name)
% [val] = block_get(B, nr, name)
%
% input:
% B    - handle to a block which properties is to be get
% name - block's property name, see function 'block'
% nr   - index of the component in the complex block (only if B is a
%        complex block)
%
% output:
% val - value of the property specified by 'name', if B is a complex block
%       val is a cell array including values of property 'name' of all its
%       components
%
% example:
%
% b = pyramid(2,1,1);
% c = block_get(b, 'BlockColor')
% creates a new pyramid and gets its color
%
% b1 = cuboid(2,1,1);
% b2 = block_move(pyramid(2,1,1,'BlockColor','r'),[0;0;1]);
% b  = block_complex(b1, b2);
% c1 = block_get(b, 'BlockColor')
% c2 = block_get(b, 2, 'BlockColor')
% creates a new complex block composed of cuboid and pyramid on its top
% and gets color of all components of the complex block (c1) and gets color
% of the pyramid - second component in the complex block (c2)
%
% See also: BLOCK, BLOCK_INFO, BLOCK_SET
%

global RTHND;

if ( ~parIsObj(B,'block') ) rethrow(geterr('argValObj', 'B', 'block')); end

nr = 0;   b = B;
switch( length(varargin) )
  case 1, name=varargin{1};
  case 2
    if ( ~parIsObj(B,'block:complex') ) rethrow(geterr('argValObj', 'B', 'block:complex')); end
    if ( ~isnumeric(varargin{1})      ) rethrow(geterr('argValNum', 'nr')); end
    nr = varargin{1};  
    if (( nr < 1 ) | ( nr > length(RTHND{B}.blocks) )) rethrow(geterr('argValObj1', 'nr')); end
    b = RTHND{B}.blocks(nr);
    name = varargin{2};
  otherwise, rethrow(geterr('argNum'));
end

try
  name = checkopt(RTHND{b}.ident,name);
  switch( RTHND{b}.ident )
    case 'block:basic',   val=blkget(b, name);
    case 'block:complex', val={}; for i=RTHND{b}.blocks val={val{:}, blkget(i, name)}; end
  end
catch rethrow(lasterror); end;    
