
function [B] = block_motion(B, T, varargin)

%BLOCK_MOTION performs animation of the blocks
% 
% syntax:
% [B] = block_motion(B, T, ...)
% [B] = block_motion(B, T, ..., delay)
% [B] = block_motion(B, T, ..., delay, trace)
%
% input:
% B       - handle to block to be animated
% T       - cell array containing 4x4 transformation matrix or 3x3 rotation
%           matrix or 3x1 position vector determining block's position 
%           and/or orientation in each animation step
% B,T,... - pairs: handle and cell array for each animated block
% delay   - pause between animation steps (in seconds), default 0
% trace   - determines method of motion's plotting 
%           0 - in each time instant only current motion's phase is visible
%           1 - all motion's phases are visible, 
%           default 0
%
% output:
% B - handle or handle's vector to an animated block/blocks
%
% examples:
%
% T={}; for i=1:100, T{i}=rp2t(rotx(i*pi/50),i*[0.1;0.1;0.1]); end
% axis([-1 11 -1 11 -1 11])
% b = pyramid(1,1,1);
% block_plot(b)
% block_motion(b, T, 0.025)
% shows animation of pyramid, it is moved from position (0,0,0) to
% (10,10,10), during its motion is rotated around x-axis by 2*pi radians
%
% T = {}; 
% for i=1:25,  T{i}=rp2t(rotz(i*pi/50),i*[0.1;0.1;0.1]); end
% for i=26:50, T{i}=rotz(i*pi/50); end
% for i=51:75, T{i}=(i-25)*[0.1;0.1;0.1]; end
% axis([-1 6 -1 6 -1 6])
% b = pyramid(1,1,1);
% block_plot(b)
% block_plot_csys(b)
% block_motion(b, T, 0.035)
% shows animation of pyramid with its coordinate system, it is moved 
% from position (0,0,0) to (2.5,2.5,2.5), during this motion it is rotated 
% around z-axis by pi/2 radians, next it is rotated around z-axis by pi/2
% radians without changing the position, next it is moved to (5,5,5)
% without changing the orientation
%
% See also: BLOCK, BLOCK_MOVE
%

global RTHND;

arg = length(varargin);
trace = 0;  delay = 0.0;
if (( arg > 1 ) & isnumeric(varargin{arg}) & isnumeric(varargin{arg-1}) )
  trace = varargin{arg};  delay = varargin{arg-1};  
  arg = arg-2; varargin = varargin(1:arg);
elseif (( arg > 0 ) & isnumeric(varargin{arg}) )
  delay = varargin{arg};  
  arg = arg-1; varargin = varargin(1:arg);
end
if ( ~parIsVector(delay,1) ) rethrow(geterr('argValNum', 'delay')); end
if ( ~parIsVector(trace,1) ) rethrow(geterr('argValNum', 'trace')); end

B = {B varargin{1:2:arg}};
for i=1:length(B)
  if ( ~parIsObj(B{i},'block') ) rethrow(geterr('argValObj', 'B', 'block')); end
  if ( RTHND{B{i}}.owner ~= 0  ) rethrow(geterr('argValObj0','B','block')); end
end

T = {T varargin{2:2:arg}};   len = 0;
for i=1:length(T)
  r = eye(3);  p = zeros(3,1);
  for j=1:length(T{i})
    if ( parIsMatrix(T{i}{j},3,1) ) p=T{i}{j}; T{i}{j}=rp2t(r,p); 
    elseif ( parIsMatrix(T{i}{j},3,3) ) r=T{i}{j}; T{i}{j}=rp2t(r,p); 
    elseif ( parIsMatrix(T{i}{j},4,4) ) r=T{i}{j}(1:3,1:3); p=T{i}{j}(1:3,4); end
  end
  if ( ~parIsCellT(T{i},0) ) rethrow(geterr('argValCellT', 'T')); end
  if ( length(T{i}) > len ) len = length(T{i}); end
end

B = cell2mat(B); fv = [];  cv = [];  blk = [];
for i=1:length(B)
  if ( RTHND{B(i)}.fig.axes ~= -1 )
    blk = [blk i];
    fv  = [fv RTHND{B(i)}.fig.view];
    cv  = [cv RTHND{B(i)}.csys.view];
  end
end;

if ( ~isempty(blk) )
  for i=blk hold(RTHND{B(i)}.fig.axes, 'on'); end
  for i=1:len
    for j=blk
      if ( i > length(T{j}) ) continue; end
      blkmv(B(j), T{j}{i}, 0);
      if ( ~trace      ) blkdel(B(j), 'all', 0); end
      if ( fv(j) ~= -1 ) blkplt(B(j), RTHND{B(j)}.fig.axes, fv(j)); end
      if ( cv(j) ~=  0 ) blkpltcsys(B(j), RTHND{B(j)}.fig.axes); end
    end
    drawnow; pause(delay)
  end
  for i=blk hold(RTHND{B(i)}.fig.axes, 'off'); end
end

for i=setdiff(1:length(B), blk)
  blkmv(B(i), T{i}{length(T{i})}, 0);
end;

if ( nargout == 0 ) clear B; end
