
function [B] = block_move(B, T, kind)

%BLOCK_MOVE changes position and/or orientation of a given block
% 
% syntax:
% [B] = block_move(B, T)
% [B] = block_move(B, T, kind)
%
% input:
% B - handle to a block to be moved
% T - 4x4 transformation matrix determining position and orientation or
%     3x3 rotation matrix or 3x1 position vector 
% kind - method for determining new position and/or orientation
%        allowed values:
%        0 - T is new position and orientation, default value
%        1 - current position and orientation transformed according to
%            dependency T*Tb
%        2 - current position and orientation transformed according to
%            dependency Tb*T
%        where: 
%        Tb-current position and orientation; T-given transformation matrix
%
% output:
% B - handle to a moved block
%
% example:
% b = pyramid(2,1,1)
% axis([-2 2 -2 2 -2 2])
% block_plot(b)
% pause
% block_move(b,rp2t(rotx(pi),[0;0;-1]))
% creates a new pyramid (base lies in xy-plane, base center is located in 
% (0,0) point), after pressing any key rotates it around x-axis by pi 
% radians and moves along z-axis by 1 unit, so the apex of pyramid is in 
% (0,0,0) point
%
% See also: RP2T, ROTX, ROTY, ROTZ
%

global RTHND;

if ( nargin < 3 ) kind=0; end
if ( parIsMatrix(T,3,1) ) T=rp2t(eye(3),T); 
elseif ( parIsMatrix(T,3,3) ) T=rp2t(T,zeros(3,1)); end

if ( ~parIsObj(B,'block')      ) rethrow(geterr('argValObj', 'B', 'block')); end
if ( RTHND{B}.owner ~= 0       ) rethrow(geterr('argValObj0','B','block')); end
if ( ~parIsMatrix(T,4,4)       ) rethrow(geterr('argValT', 'T')); end
if (( kind > 2 ) | ( kind < 0 )) rethrow(geterr('argVal', '0, 1, 2')); end

B = blkmv(B, T, kind);
if ( RTHND{B}.fig.axes ~= -1 )
  hold(RTHND{B}.fig.axes, 'on');
  B = blkrdrw(B,1);
  hold(RTHND{B}.fig.axes, 'off');
end

if ( nargout == 0 ) clear B; end
