
function [CS]=csys(T, varargin)

%CSYS creates a new coordinate system defined by 4x4 transformation matrix
%     or rotation matrix, or position vector
% 
% syntax:
% [CS] = csys(T)
% [CS] = csys(T, 'PropertyName', PropertyValue, ...)
%
% input:
% T - 4x4 transformation matrix determining position and orientation or
%     3x3 rotation matrix or 3x1 position vector 
% property name/property value pairs are used to specify additional 
% properties
%
% property list:
% CSysAxisLabel  - cell array, labels of the axis of the coordinate system,
%                  default {'x', 'y', 'z'}
% CSysAxisLength - x, y, and z limits of the coordinate system,
%                  default [-1 1 -1 1 -1 1]
% CSysColor      - axis' color determined according to Matlab color 
%                  specification, default 'k'
% CSysLabel      - description of the coordinate system, 
%                  default empty string
% CSysLabelColor - color of all descriptions of the coordinate system
%                  determined according to Matlab color specification, 
%                  default 'k'
% CSysWidth      - width (in points) of the axis, default 0.5
% 
% output:
% CS - handle to a new coordinate system
%
% examples:
%
% axis([-3 3 -3 3 -3 3]);
% cs = csys(rp2t(rotz(pi/4),[0;0;0]))
% csys_plot(cs)
% creates and plots a coordinate system, its origin is located in (0,0,0) 
% point, system is rotated around z-axis by pi/4 radians, all properties 
% are default
%
% axis([-3 3 -3 3 -3 3]);
% cs = csys([0;0;0], 'CSysAxisLength', axis)
% csys_plot(cs)
% creates and plots global coordinate system
%
% See also: RP2T, ROTX, ROTY, ROTZ,
%           CSYS_GET, CSYS_SET, CSYS_INFO
%           CSYS_PLOT, CSYS_DELETE
%

global RTHND;

if ( parIsMatrix(T,3,1) ) T=rp2t(eye(3),T); 
elseif ( parIsMatrix(T,3,3) ) T=rp2t(T,zeros(3,1)); end

if ( ~parIsMatrix(T,4,4) ) rethrow(geterr('argValT', 'T')); end

try   lst = prepopts('csys', varargin);
catch rethrow(lasterror); end;

cs = cscreate(T, lst);

CS        = gethnd;
cs.hnd    = CS;
RTHND{CS} = cs;
