
function [CS] = csys_move(CS, T, kind)

%CSYS_MOVE changes position and/or orientation of a given coordinate system
% 
% syntax:
% [CS] = csys_move(CS, T)
% [CS] = csys_move(CS, T, kind)
%
% input:
% CS   - handle to a coordinate system to be moved
% T    - 4x4 transformation matrix determining position and orientation or
%        3x3 rotation matrix or 3x1 position vector 
% kind - method for determining new position and/or orientation
%        allowed values:
%        0 - T is new position and orientation, default value
%        1 - T describes changes of position and orientation according to 
%            global coordinate system (new position and orientation 
%            is caclulated using dependency T*Tcs)
%        2 - T describes changes of position and orientation according to 
%            current coordinate system (new position and orientation 
%            is caclulated using dependency Tcs*T)
%        where: 
%        Tcs-current position and orientation; T-given transformation matrix
%
% output:
% CS - handle to a moved coordinate system 
%
% example:
% axis([-3 3 -3 3 -3 3]);
% cs = csys(eye(4))
% csys_plot(cs)
% pause
% csys_move(cs,rp2t(rotx(pi),[2;-2;-1]))
% creates a new coordinate system (coinciding with the global coordinate 
% system), after pressing any key rotates it around x-axis by pi radians 
% and moves its origin to point [2;-2;-1]
%
% See also: CSYS, RP2T, ROTX, ROTY, ROTZ
%

global RTHND;

if ( nargin < 3 ) kind=0; end
if ( parIsMatrix(T,3,1) ) T=rp2t(eye(3),T); 
elseif ( parIsMatrix(T,3,3) ) T=rp2t(T,zeros(3,1)); end

if ( ~parIsObj(CS,'csys')      ) rethrow(geterr('argValObj', 'CS', 'csys')); end
if ( ~parIsMatrix(T,4,4)       ) rethrow(geterr('argValT', 'T')); end
if (( kind > 2 ) | ( kind < 0 )) rethrow(geterr('argVal', '0, 1, 2')); end

CS = csmv(CS, T, kind);
if ( RTHND{CS}.fig.axes ~= -1 )
  hold(RTHND{CS}.fig.axes, 'on');
  CS = csrdrw(CS,1);
  hold(RTHND{CS}.fig.axes, 'off');
end

if ( nargout == 0 ) clear CS; end
