
function [R] = robot_catch(R, B)

%ROBOT_CATCH 'catches' a block, i.e. connects a block with a robot
% 
% syntax:
% [R] = robot_catch(R, B)
% 
% Function places a block in grasper coordinate system, a robot's
% movement will cause displacement of the block.
%
% input:
% R - handle to structure representing a robot
% B - handle to block to catch
%
% output:
% R - handle to structure representing a robot with 'catched' block
%
% example:
% axis([-1 5 -1 5 -3 3]);
% campos([-1 -1 3]);
% camlight
% r = robot([dh(0,0,0,0)], ['r'], 'RobotCylinderRadius', 0.5, ...
%     'RobotGrasper', [3;0;0], 'RobotGrasperSize', [1,2]);
% robot_plot(r, 'surf');
% pause
% b = block_move(cube(1,'BlockColor','g'),[4;0;-0.4]);
% block_plot(b,'surf');
% robot_catch(r,b);
% pause
% robot_motion(r,[0:0.05:pi/2]);
% creates a robot with one revolute joint, after pressing any key creates 
% a block which is catched by robot, after pressing any key a robot moves 
% to new configuration moving a block
%
% See also: ROBOT, ROBOT_DROP, ROBOT_CONFIG, ROBOT_MOTION, 
%

global RTHND;

if ( ~parIsObj(R,'robot:manip') ) rethrow(geterr('argValObj', 'R', 'robot:manip')); end
if ( ~parIsObj(B,'block') ) rethrow(geterr('argValObj', 'B', 'block')); end

T = trans2(RTHND{R}.dh, RTHND{R}.base);
RTHND{R}.tblock = invt(T{RTHND{R}.n}*RTHND{R}.grasper) * RTHND{B}.T;
RTHND{R}.cblock = B;
RTHND{B}.owner  = R;
