
function [R] = robot_complex(varargin)

%ROBOT_COMPLEX creates complex robot from given robots or
%              connects given robots to existing complex robot
% 
% syntax:
% [R] = robot(R, ...)
% [R] = robot(R, ..., 'PropertyName', PropertyValue, ...)
%
% A new robot is an union of few robots, each component is a separate 
% robot, it has its own handle and properties. In default views all 
% components are connected by lines/cylindres. Created complex robot has 
% own properties, which define appearance of the connection.
%
% input:
% R, ... - a list of handles of robots to be connected 
% property name/property value pairs are used to specify additional robot
% properties
%
% property list:
%
% RobotBase  - 4x4 transformation matrix or 3x3 rotation matrix 
%              or 3x1 position vector determining position and/or
%              orientation of robot's base, default it is equal to
%              the base of the first robot in the list
%
% RobotLineWidth      - width (in points) of the lines in 'line' view
% RobotCylinderRadius - radius of cylinders representing connections in
%                       default 3D view (see: robot_plot), default 1
% RobotCylinderApprox - approximation of cylinders representing connections
%                       in default 3D view (see: robot_plot), cylinders are 
%                       approximated by right prisms, value specifies number 
%                       of base's edges, default 8
%
% RobotColor         - 2 elements cell array or single value,
%                      robot's color determined according to Matlab color 
%                      specification, used in default views; 
%                      if parameter is a cell array individual elements 
%                      correspond to properties: RobotLengthColor and 
%                      RobotExColor; if parameter is a single value defines 
%                      values of properties: RobotLengthColor, RobotExColor;
%                      default 'b'
% RobotLengthColor   - color of objects representing connections, 
%                      used in default views, default 'b'
% RobotExColor       - color of objects representing additional elements
%                      of the connections (see: RobotModel), used in default
%                      3D view, default 'b'
%
% RobotEdgeColor         - 2 elements cell array or single value,
%                          edges' color determined according to Matlab color 
%                          specification, used only in 'surf' view;
%                          if parameter is a cell array individual elements 
%                          correspond to properties: RobotLengthEdgeColor 
%                          and RobotExEdgeColor; if parameter is a single 
%                          value defines values of properties:  
%                          RobotLengthEdgeColor and RobotExEdgeColor, 
%                          default 'none'
% RobotLengthEdgeColor   - edges' color of objects representing connections, 
%                          used only in 'surf' views, default 'none'
% RobotExEdgeColor       - edges' color of objects representing additional elements
%                          of the connections (see: RobotModel), used in default 3D view,
%                          default 'none'
% RobotEdgeWidth         - width (in points) of the edges in 'mesh' or 
%                          'surf' view, default 0.5 
%
% RobotModel - string, method of plotting the complex robot in default 3D view,
%              allowed values: 
%              'config', 'range' - conections between components are
%                         cylinders
%              'configex', 'rangeex' - same as 'config' and 'range' but 
%                         each connection is marked out by an additional
%                         sphere (a sphere conected to the base of the
%                         complex robot is grater then others)
%              default 'config'
%
% CSysAxisLabel  - cell array, labels of the axis of the robot's coordinate 
%                  system, default {'x', 'y', 'z'}
% CSysAxisLength - x, y, and z limits of the robot's coordinate system,
%                  default [0 1 0 1 0 1]
% CSysColor      - robot's coordinate system color determined according to 
%                  Matlab color specification, default 'r'
% CSysLabel      - description of the robot's coordinate system, 
%                  default 'C'
% CSysLabelColor - color of all descriptions of the robot's coordinate 
%                  system, determined according to Matlab color specification, 
%                  default 'k'
% CSysWidth      - width (in points) of the axis, default 0.5
% 
% output:
% R - handle to structure representing complex robot
%
% examples:
%
% r1 = robot( [dh(0,0,0,-pi/2); dh(0,4,0,-pi/2); dh(0,3,0,-pi/2)], ['r', 'r', 'r'],...
%             'RobotBase', rp2t(rotx(-pi/2),[2;2;0]),...
%             'RobotGrasper', [2; 0; 0], 'RobotGrasperSize', 1);
% r2 = robot( [dh(0,0,0,-pi/4); dh(0,4,0,-pi/2); dh(0,3,0,-pi/4)], ['r', 'r', 'r'],...
%             'RobotBase', rp2t(rotx(-pi/2),[2;-2;0]),...
%             'RobotGrasper', [2; 0; 0], 'RobotGrasperSize', 1);
% r = robot_complex(r1, r2);
% axis([-3 7 -5 5 0 10]);
% robot_plot(r);
% view([-45,40])
% creates and plots a complex robot consisting of two planar manipulators 
% with three revolute kinematics pairs, the base of the complex robot is 
% equal to the base of the first manipulator, all properties are default
%
% r = robot_complex();
% r1 = robot( [dh(0,0,0,-pi/2); dh(0,4,0,-pi/2); dh(0,3,0,-pi/2)], ['r', 'r', 'r'],...
%             'RobotBase', rp2t(rotx(-pi/2),[2;2;0]),...
%             'RobotGrasper', [2; 0; 0], 'RobotGrasperSize', 1);
% r2 = robot( [dh(0,0,0,-pi/4); dh(0,4,0,-pi/2); dh(0,3,0,-pi/4)], ['r', 'r', 'r'],...
%             'RobotBase', rp2t(rotx(-pi/2),[2;-2;0]),...
%             'RobotGrasper', [2; 0; 0], 'RobotGrasperSize', 1);
% r = robot_complex(r, r1, r2);
% axis([-3 7 -5 5 0 10]);
% robot_plot(r);
% view([-10,45])
% creates an empty complex robot, next attaches two planar manipulators 
% with three revolute kinematics pairs and plots the robot, the base of 
% the complex robot corresponds to the global coordinate system, 
% all properties are default
%
% See also: ROBOT, ROBOT_CONNECT
%

global RTHND;

i = 1;
while(( i <= length(varargin) ) & parIsObj(varargin{i},'robot') ) i=i+1; end
h = varargin(1:i-1);
varargin = varargin(i:length(varargin));

try   [lst, setlst] = prepopts('robot:complex', varargin);
catch rethrow(lasterror); end;
  
if ( length(h) > 0 )

  for i=1:length(h)
    r = h{i};
    if ( ~parIsObj(r,'robot') ) rethrow(geterr('argValObj','R','robot')); end
    if ( RTHND{r}.owner & parIsObj(RTHND{r}.owner,'robot') ) rethrow(geterr('argValObj0','R','robot')); end
  end

  h = cell2mat(h);  R = h(1);
  if ( ~parIsObj(R,'robot:complex') )
    R = gethnd;
    r = rcmplxcreate(R, h, lst);
    RTHND{R} = r;
  else
    if ( length(h) > 1 ) RTHND{R}=rcmplxattach(RTHND{R},h(2:length(h)),0); end
    for i=1:size(setlst,1) 
      RTHND{R} = rset(RTHND{R}, setlst{i,1}, setlst{i,2}, setlst{i,4}, 0); 
    end
  end

  owner = getowner(R, 'hnd');
  axs = RTHND{owner}.fig.axes;
  if ( axs ~= -1 )
    fhndadd(R);
    hold(axs, 'on');
    R = rrdrw(R, axs, 1);
    hold(axs, 'off');
  end
  pnl = RTHND{owner}.panel;
  if ( pnl ) close(pnl);  rpnl('init', owner, figure); end
    
  
else 

  R = gethnd;
  r = rcmplxcreate(R, [], lst);
  RTHND{R} = r;
  
end
    
if ( nargout == 0 ) clear R; end
