
function [val] = robot_config(R, q)

%ROBOT_CONFIG changes or returns current configuration of the robot
% 
% syntax:
% [R] = robot_config(R, q)
% [q] = robot_config(R)
%
% input:
% R - handle to structure representing a robot
% q - vector, new configuration of the robot (*); 
%     if length of configuration vector is less then number of 
%     configuration variables then only the starting variables are set
%
% output:
% R - handle to structure representing a robot
% q - vector, current configuration of the robot (*)
%
% (*) In the case of connected mechanisms, defined by robot_complex and/or 
% robot_connect, single configuration is described by configurations of all 
% components. Configuration vector contains configuration of each component
% robot in the order of the structure of the mechanism. See example II.
%
% examples:
%
% axis([-5 10 -5 5 -2 10]);
% r = robot([dh(0,0,0,-pi/4); dh(0,5,0,pi/2); dh(0,5,0,-pi/4)], ['r','r','r'], ...
%           'RobotBase', rotx(-pi/2), 'RobotGrasper', [2; 0; 0]);
% robot_plot(r)
% pause
% robot_config(r, [-pi/2; pi/2; pi/2]);
% conf1 = robot_config(r)
% pause
% robot_config(r, -3*pi/4)
% conf2 = robot_config(r)
% creates a planar manipulator with three revolute kinematics pairs, 
% after pressing any key sets its configuration to [-pi/2; pi/2; pi/2] 
% and gets current configuration, next after pressing any key sets the 
% configuration to [-3pi/4; pi/2; pi/2] (only first configuration variable 
% is changed)
%
% axis([-3 7 -5 5 0 10]);
% view([-68,44])
% r1 = robot( [dh(0,0,0,-pi/2); dh(0,4,0,-pi/2)], ['r', 'r'],...
%             'RobotBase', rp2t(rotx(-pi/2),[2;2;0]), 'RobotColor', 'c', ...
%             'RobotGrasper', [2; 0; 0], 'RobotGrasperSize', 1);
% r2 = robot( [dh(0,0,0,-pi/4); dh(0,4,0,0)], ['r', 'r'],...
%             'RobotBase', rp2t(rotx(-pi/2),[2;-2;0]), 'RobotColor', 'r');
% r3 = robot( [dh(0,0,0,-pi/4)], ['r'], 'RobotBase', [0;0;-1], ...
%             'RobotGrasper', [2;0;0], 'RobotColor', 'g');  
% r4 = robot( [dh(0,0,0,-pi/4)], ['r'], 'RobotBase', [0;0;1], ...
%             'RobotGrasper', [2;0;0], 'RobotColor', 'm'); 
% robot_connect(r2, r3, r4);
% r = robot_complex(r2, r1);
% robot_plot(r)
% pause
% robot_config(r, [-pi/2; 0; -pi/2; 0; 0; 0])
% conf = robot_config(r)
% creates two planar manipulators with two revolute joints (r1 - cyan, 
% r2 - red), next creates two planar manipulators with one revolute joint 
% (r3 - green, r4 - magenta); r3 and r4 are attached to the robot r2; 
% r1 and r2 are connected into the one mechanism r; 
% after pressing any key the configuration of r is set to
% [-pi/2;0;-pi/2;0;0;0]; r is a robot_complex containing r2 and r1 (r2 has
% been given as the first), moreover r3 and r4 have been attached to r2, so
% configutation vector of whole mechanism containes a sequence
% configurations of robots: r2 (2 configuration variables), 
% r3 (1 configuration variable), r4 (1 configuration variable) and 
% r1 (2 configuration variables)
%
% See also: ROBOT, ROBOT_MOTION
%

global RTHND;

if ( ~parIsObj(R,'robot') ) rethrow(geterr('argValObj', 'R', 'robot')); end

if ( nargin == 1 )
    
  val = rcfgget(R, 1);
  
else
    
  range = rcfgget(R, 2);
  if ( size(q,2) > 1 ) q=q'; end
  if (( size(q,2) ~= 1 ) | ~isnumeric(q) | ( size(q,1) > size(range,1) )) 
    rethrow(geterr('argValVect', 'q', size(range,1))); 
  end
  for i=1:length(q)
    if ( q(i) < range(i,1) ) 
      q(i) = range(i,1);
      warning('ROBOT:robotRange2', getwrn('robotRange2', i, q(i)))
    elseif ( q(i) > range(i,2) ) 
      q(i) = range(i,2);
      warning('ROBOT:robotRange2', getwrn('robotRange2', i, q(i)))
    end
  end
  
  rcfgset(R, q);  val=R;
  
  axs = getowner(R, 'axes'); 
  if ( axs ~= -1 )
    hold(axs, 'on');
    R = rrdrw(R, axs, 1);
    hold(axs, 'off');
  end
  if ( nargout == 0 ) clear val; end

end
