
function [R] = robot_connect(R, varargin)

%ROBOT_CONNECT connects one complex robot or few manipulators to the 
%              existing robot
% 
% syntax:
% [R] = robot(R, R1, ...)
%
% The attached robots are connected to the last segment of the existing 
% robot. If the base of the attatched robot does not correspond to 
% coordinate system of the last segment, additional connector is added
% in default view.
%
% input:
% R       - handle to structure representing an existing robot
% R1, ... - a list of handles of robots to be connected 
%
% output:
% R - handle to structure representing modified robot
%
% examples:
% 
% r = robot( [dh(0,0,0,-pi/4); dh(0,4,0,-pi/4); dh(0,3,0,-pi/4)], ['r', 'r', 'r'],...
%            'RobotBase', rp2t(rotx(-pi/2),[1;1;0]));
% r1 = robot( [dh(0,0,0,0); dh(0,2,0,-pi/2); dh(0,1,0,-pi/4)], ['r', 'r', 'r'],...
%             'RobotBase', [0;0;1], 'RobotGrasper', [1; 0; 0]);
% r2 = robot( [dh(0,0,0,0); dh(0,2,0,-pi/2); dh(0,1,0,-pi/4)], ['r', 'r', 'r'],...
%             'RobotBase', [0;0;-1], 'RobotGrasper', [1; 0; 0]);
% robot_connect(r,r1,r2)
% axis([0 8 -6 3 0 9]);
% robot_plot(r);
% view([50,20])
% creates three planar manipulators with three revolute kinematics pairs, 
% next the manipulators r1, r2 are connected to manipulator r, in this way
% mechanism like a hand with two fingers (r1, r2) is created
% 
% r = robot( [dh(0,0,0,-pi/4); dh(0,4,0,-pi/4); dh(0,3,0,-pi/4)], ['r', 'r', 'r'],...
%            'RobotBase', rp2t(rotx(-pi/2),[1;1;0]));
% r1 = robot( [dh(0,0,0,0); dh(0,2,0,-pi/2); dh(0,1,0,-pi/4)], ['r', 'r', 'r'],...
%             'RobotBase', [0;0;1], 'RobotGrasper', [1; 0; 0]);
% r2 = robot( [dh(0,0,0,0); dh(0,2,0,-pi/2); dh(0,1,0,-pi/4)], ['r', 'r', 'r'],...
%             'RobotBase', [0;0;-1], 'RobotGrasper', [1; 0; 0]);
% r12 = robot_complex(r1,r2);
% robot_connect(r,r12)
% axis([0 8 -6 3 0 9]);
% robot_plot(r);
% view([50,20])
% creates the planar manipulator with three revolute kinematics pairs, 
% and robot complex consisting of two planar manipulators, next the complex
% robot is connected to the first manipulator, in this way the same
% mechanism as in the first example is created
% 

global RTHND;

if ( parIsObj(R,'robot:complex') ) rethrow(geterr('argValObj','R','robot:manip, robot:platform')); end

if ( length(varargin) > 0 )

  for i=1:length(varargin)
    r = varargin{i};
    if ( ~parIsObj(r,'robot') ) rethrow(geterr('argValObj','R','robot')); end
    if ( RTHND{r}.owner & parIsObj(RTHND{r}.owner,'robot') ) rethrow(geterr('argValObj0','R','robot')); end
  end

  r = varargin{1};
  
  if ( RTHND{R}.bond & parIsObj(r,'robot:complex') ) rethrow(geterr('argValObj','R','robot:manip')); end
  
  if ( parIsObj(r,'robot:manip') )
    cr = 0;
    for r=cell2mat(varargin)
      if ( ~parIsObj(r,'robot:manip') ) rethrow(geterr('argValObj','R','robot:manip')); end
      cr = max(cr, RTHND{r}.def3d.cylinderRadius);
    end
    if ( RTHND{R}.bond )
      r = RTHND{R}.bond;
      RTHND{r} = rcmplxattach(RTHND{r}, cell2mat(varargin) ,0);
    else
      r  = gethnd();
      rc = rcmplxcreate(r, cell2mat(varargin),...
           prepopts('robot:complex', {'RobotBase', eye(4), 'RobotCylinderRadius',cr}));
      RTHND{r} = rc;
    end
    varargin = {};
  end

  if ( length(varargin) > 1 ) warning('ROBOT:robotUnion', getwrn('robotUnion')); end
  fhnddel(r);  rdel(r,'all',1);
  RTHND{R}.bond  = r; 
  RTHND{r}.owner = R;
  
  RTHND{r} = rrfrsh(RTHND{r}, 'RobotModel');
  
  owner = getowner(R, 'hnd');  
  axs   = RTHND{owner}.fig.axes;
  if ( axs ~= -1 )
    hold(axs, 'on');
    R = rrdrw(R, axs, 1);
    hold(axs, 'off');
  end
  pnl = RTHND{owner}.panel;
  if ( pnl ) close(pnl);  rpnl('init', owner, figure); end
    
end
    
if ( nargout == 0 ) clear R; end
