
function [R] = robot_delete(R, opt)

%ROBOT_DELETE deletes the robot and/or associated coordinates systems
%             from figure window
% 
% syntax:
% [R] = robot_delete(R)
% [R] = robot_delete(R, opt)
%
% Function removes graphical representation of the robot and/or 
% coordinates systems connected to its joints from figure window.
%
% input:
% R   - handle to robot to be deleted
% opt - determines deleted element, allowed values:
%       'all'   - deletes robot and coordinates systems
%       'robot' - deletes only robot
%       'csys'  - deletes only coordinates systems
%       default value 'all'
%
% output:
% R - handle to structure representing a robot
%
% example:
% axis([-5 15 -10 10 -5 10]);
% r = robot([dh(0,0,0,-pi/4); dh(0,5,0,pi/2); dh(0,5,0,-pi/4)], ['r','r','r'], ...
%           'RobotBase', rp2t(rotx(-pi/2),[0;0;0]), 'RobotGrasper', [3; 0; 0]);
% robot_plot(r);
% robot_plot_csys(r);
% pause
% robot_delete(r, 'csys');
% creates and plots a planar manipulator with three revolute kinematics pairs 
% and coordinates systems connected to robot joints, after pressing any key 
% coordinates systems are removed
%
% See also: ROBOT, ROBOT_PLOT, ROBOT_PLOT_CSYS
%

global RTHND;

if ( nargin < 2 ) opt='all'; end

if ( ~parIsObj(R,'robot') ) rethrow(geterr('argValObj', 'R', 'robot')); end
if ( ~parIsVectorTxt(opt) ) rethrow(geterr('argValTxt', 'opt')); end
if ( ~strcmpi(opt,'all') & ~strcmpi(opt,'robot') & ~strcmpi(opt,'csys') ) 
  rethrow(geterr('argVal','opt','all, csys, robot')); 
end
if ( ~strcmpi(opt, 'csys') & RTHND{R}.owner  & parIsObj(RTHND{R}.owner,'robot') )
  rethrow(geterr('argValObj0','R','robot')); 
end

fhnddel(R);
rdel(R, opt, 1);

if ( nargout == 0 ) clear R; end
