
function [p, T] = robot_fkinematics(R, joint, point)

%ROBOT_FKINEMATICS calculates forward kinematics of robot
% 
% syntax:
% [p, T] = robot_fkinematics(R)
% [p, T] = robot_fkinematics(R, joint, point)
%
% input:
% R     - handle to structure representing a robot
% joint - vector containing numbers of robot's joint,
%         default last joint
% point - 3-row matrix containing coordinates of points  
%         in local coordinates of joints,
%         default grasper position
%
% output:
% p - 3-row matrix, coordinates of points in global coordinate system
% T - cell array, transformation matrices describing position and orientation
%     of local coordinate systems connected with each joint
%
% examples:
%
% r = robot([dh(0,0,0,-pi/4); dh(0,5,0,pi/2); dh(0,5,0,-pi/4)],...
%           ['r','r','r'], 'RobotGrasper', [3;0;0]);
% [p1]   = robot_fkinematics(r)
% [p2,T] = robot_fkinematics(r,[1,2],[[5;0;0],[5;0;0]])
% creates a planar manipulator with three revolute kinematics pairs, and:
% 1. calculates position of grasper (p1),
% 2. calculates positions of endpoints of first and second joint (p2) 
%    and transformation matrices describing these joints (T)
%
% r1 = robot( [dh(0,0,0,pi/2); dh(0,4,0,-pi/2)], ['r', 'r'],...
%             'RobotColor', 'r', 'RobotGrasper', [2;0;0]);
% r2 = robot( [dh(0,0,0,pi/4); dh(0,4,0,-pi/2)], ['r', 'r'],...
%             'RobotColor', 'b', 'RobotGrasper', [2;0;0]);
% r  = robot_complex(r1, r2);
% p  = robot_fkinematics(r)
% creates a mechanism consisting of two planar manipulators with two
% revolute joints, next calculates positions of their graspers
%
% See also: ROBOT, ROBOT_INFO
%

global RTHND;

if ( ~parIsObj(R,'robot') ) rethrow(geterr('argValObj', 'R', 'robot')); end
if ( nargin > 1 )
  if ( size(point,1) ~= 3             ) rethrow(geterr('argValMtx1', point, 3)); end
  if ( size(point,2) ~= length(joint) ) rethrow(geterr('argValMtx2', point, length(joint))); end
end;

[t, nr, g] = rfkin(RTHND{R}, eye(4));

if ( nargin == 1 ) 
  joint = nr;  point = g; 
end;

point = [point;ones(1,size(point,2))];
p     = [];
for i=1:length(joint)
  p = [p t{joint(i)}*point(:,i)];
end
p = p(1:3,:);

if ( nargout > 1 ) T = { t{joint} }; end

end
