
function [val] = robot_get(R, varargin)

%ROBOT_GET returns the value of the robot's property
% 
% syntax:
% [val] = robot_get(R, name)
% [val] = robot_get(R, nr, name)
%
% input:
% R    - handle to structure representing a robot
% name - robot's property name, see function 'robot'
% nr   - index of the robot in the complex robot (only if R is a
%        complex robot)
%
% output:
% val - value of the property specified by 'name'
%
% examples:
%
% r = robot([dh(0,0,0,-pi/4); dh(0,5,0,pi/2); dh(0,5,0,-pi/4)], ['r','r','r']);
% c = robot_get(r, 'RobotColor')
% creates a planar manipulator with three revolute kinematics pairs 
% and gets its current color
%
% r1 = robot( [dh(0,0,0,-pi/2); dh(0,4,0,-pi/2)], ['r', 'r'], 'RobotColor', 'r');
% r2 = robot( [dh(0,0,0,-pi/4); dh(0,4,0,-pi/2)], ['r', 'r'], 'RobotColor', 'b');
% r  = robot_complex(r1, r2, 'RobotColor', 'g');
% c1 = robot_get(r, 1, 'RobotColor')
% c  = robot_get(r, 'RobotColor')
% creates a mechanism consisting of two planar manipulators with two
% revolute joints, next gets color of the first manipulator (variable c1)
% and gets color of complex robot (variable c)
%
% See also: ROBOT, ROBOT_SET, ROBOT_INFO
%

global RTHND;

if ( ~parIsObj(R,'robot') ) rethrow(geterr('argValObj', 'R', 'robot')); end

nr = 0;   r = R;
switch( length(varargin) )
  case 1, name=varargin{1};
  case 2
    if ( parIsObj(R,'robot:manip') ) rethrow(geterr('argValObj', 'R', 'robot:complex')); end
    if ( ~isnumeric(varargin{1})   ) rethrow(geterr('argValNum', 'nr')); end
    nr = varargin{1};  
    if (( nr < 1 ) | ( nr > length(RTHND{R}.robots) )) rethrow(geterr('argValObj1', 'nr')); end
    r = RTHND{R}.robots(nr);
    name = varargin{2};
  otherwise, rethrow(geterr('argNum'));
end

try val=rget(r, checkopt(RTHND{r}.ident,name));
catch rethrow(lasterror); end;    
