
function [R] = robot_motion(R, q, varargin)

%ROBOT_MOTION shows robot movement according to the given trajectory
% 
% syntax:
% [R] = robot_motion(R, q, ...)
% [R] = robot_motion(R, q, ..., delay)
% [R] = robot_motion(R, q, ..., delay, trace)
%
% input:
% R       - handle to structure representing a robot
% q       - matrix with number of rows equals number of configuration 
%           variables, each column corresponds to single configuration;
%           see robot_config for more complete description for connected 
%           mechanisms
% R,q,... - pairs: handle and matrix for each animated robot
% delay   - pause between animation steps (in seconds), default 0
% trace   - determines method of motion's plotting 
%           0 - in each time instant only one configuration is visible
%           1 - all configurations given by "q" are visible, 
%           default 0
%
% output:
% R - handle or handle's vector to animated a robot/robots
%
% examples:
%
% axis([-5 5 -5 5 -5 5]);
% view([45, 40])
% camlight
% r = robot([dh(0,0,0,0)], ['r'], 'RobotCylinderRadius', 0.5, ...
%     'RobotGrasper', [3;0;0], 'RobotGrasperSize', [1,2]);
% robot_plot(r, 'surf');
% robot_motion(r,[0:0.05:pi]);
% creates a robot with one revolute joint and shows the motion of the robot 
% from configuration 0 to pi radians
%
% axis([-5 5 -5 5 -5 5]);
% view([45, 40])
% camlight
% r1 = robot([dh(0,0,0,0)], ['r'], 'RobotCylinderRadius', 0.5, ...
%     'RobotGrasper', [3;0;0], 'RobotGrasperSize', [1,2]);
% r2 = robot([dh(0,0,0,0)], ['r'], 'RobotCylinderRadius', 0.5, 'RobotBase', [0;0;2], ...
%     'RobotGrasper', [3;0;0], 'RobotGrasperSize', [1,2]);
% robot_plot(r1, 'surf');
% robot_plot(r2, 'surf');
% robot_motion(r1,linspace(0,pi,20),r2,linspace(0,pi/2,10),0.05);
% creates two robots with one revolute joint and shows their motion 
% from configuration 0 to pi radians and 0 to p/2 radians respectively
%
% axis([-3 7 -5 5 0 10]);
% view([-68,44])
% r1 = robot( [dh(0,0,0,-pi/2); dh(0,4,0,-pi/2)], ['r', 'r'],...
%             'RobotBase', rp2t(rotx(-pi/2),[2;2;0]), 'RobotColor', 'c', ...
%             'RobotGrasper', [2; 0; 0], 'RobotGrasperSize', 1);
% r2 = robot( [dh(0,0,0,-pi/4); dh(0,4,0,0)], ['r', 'r'],...
%             'RobotBase', rp2t(rotx(-pi/2),[2;-2;0]), 'RobotColor', 'r');
% r3 = robot( [dh(0,0,0,-pi/4)], ['r'], 'RobotBase', [0;0;-1], ...
%             'RobotGrasper', [2;0;0], 'RobotColor', 'g');  
% r4 = robot( [dh(0,0,0,-pi/4)], ['r'], 'RobotBase', [0;0;1], ...
%             'RobotGrasper', [2;0;0], 'RobotColor', 'm'); 
% robot_connect(r2, r3, r4);
% r = robot_complex(r2, r1);
% robot_plot(r);
% dq = pi/25;
% pause
% q1 = [linspace(-pi/2,0,10); linspace(-pi/2,0,10)];
% q2 = linspace(-pi/4,-pi/2,10);
% q3 = linspace(-pi/4,-pi/2,10);
% q4 = linspace(-pi/4,0,10);
% robot_motion(r, [q2; zeros(1,length(q2)); q3; q4; q1], 0.1)
% pause
% q2 = linspace(-pi/2,-pi/4,10);
% q3 = linspace(-pi/2,-pi/4,10);
% robot_motion(r, [q2; zeros(1,length(q2)); q3], 0.1)
% pause
% q1 = [linspace(0,-pi/2,10); linspace(0,-pi/2,10)];
% robot_motion(r1, q1, 0.1)
% creates two planar manipulators with two revolute joints (r1 - cyan, 
% r2 - red), next creates two planar manipulators with one revolute joint 
% (r3 - green, r4 - magenta); r3 and r4 are attached to the robot r2; 
% r1 and r2 are connected into the one mechanism r; 
% after pressing any key the motion of mechanism r is performed:
% sub-manipulator r1 is moved to configuration [0;0], r2 is moved to
% [-pi/2;0] and connected manipulators r3 and r4 are moved to -pi/2, 0
% respectively; 
% after pressing any key a part of mechanism r is moved: manipulators r2 
% is moved to configuration [-pi/4;0] and r3 to configuration -pi/4;
% after pressing any key manipulator r1 is moved to configuration
% [-pi/2;-pi/2]
%
% See also: ROBOT, ROBOT_CONFIG
%

global RTHND;

len = length(rcfgget(R, 1));
if ( ~parIsObj(R,'robot')  ) rethrow(geterr('argValObj', 'R', 'robot')); end
if ( ~isnumeric(q) | ( size(q,1) > len )) rethrow(geterr('argValMtx1', 'q', len)); end

q = {q};
for i=1:2:length(varargin)
  r = varargin{i};
  if ( parIsObj(r,'robot') ) 
    if ( i < length(varargin) )
      len = length(rcfgget(r,1));
      trj = varargin{i+1};
      if ( ~isnumeric(trj) | ( size(trj,1) > len )) rethrow(geterr('argValMtx1', 'q', len)); end
      R = [R r];
      q = {q{:} trj};
    else rethrow(geterr('argRMtn')); end
  else break; end
end

varargin = varargin(2*(length(R)-1)+1:length(varargin));
arg = length(varargin);
trace = 0;  delay = 0.0;  
if ( arg > 1 ) trace = varargin{arg};  delay = varargin{arg-1};  
elseif ( arg > 0 ) delay = varargin{arg}; end
if ( ~parIsVector(delay,1) ) rethrow(geterr('argValNum', 'delay')); end
if ( ~parIsVector(trace,1) ) rethrow(geterr('argValNum', 'trace')); end

% disp('start')
% for i=1:length(R)
%   rng = rcfgget(R(i), 2);
%   for j=1:size(q{i},2)
%     if ( any(q{i}(:,j)<rng(:,1)) )
%       v = find(q{i}(:,j)<rng(:,1));
%       c=''; for k=v c=[c, sprintf('%d ',v)]; end
%       disp(sprintf('Przekroczony dolny zakres - zmienna/e: %s, konfiguracja numer: %d',c,j));
%       rng(v,1)
%       q{i}(v,j)
%       return
%     end
%     if ( any(q{i}(:,j)>rng(:,2)) )
%       v = find(q{i}(:,j)>rng(:,2));
%       c=''; for k=v c=[c, sprintf('%d ',v)]; end
%       disp(sprintf('Przekroczony grny zakres - zmienna/e: %s, konfiguracja numer: %d',c,j));
%       rng(v,2)
%       q{i}(v,j)
%       return
%     end
%   end
% end
  

len = 0;
for i=1:length(q) len = max(len, size(q{i},2)); end

r = [];  axs = [];
for i=1:length(R)
  if ( RTHND{R(i)}.fig.axes ~= -1 ) 
    r = [r i]; 
    axs = [axs, getowner(R(i), 'axes')]; 
  end
end;

if ( ~isempty(r) )
  for i=r hold(axs(i), 'on'); end
  for i=1:len
    for j=r
      if ( i > size(q{j},2) ) continue; end
      rcfgset(R(j), q{j}(:,i));
      rrdrw(R(j), axs(j), ~trace);
    end
    drawnow; pause(delay)
  end
  for i=r hold(axs(i), 'off'); end
end

for i=setdiff(1:length(R), r)
  rcfgset(R(i), q{i}(:,size(q{i},2)));
end;

if ( nargout == 0 ) clear R; end

% disp('koniec')
