
function [R] = robot_part(R, varargin)

%ROBOT_PART defines the apperance of robot or 
%           returns list of blocks defining apperance
% 
% syntax for single-segment robots:
% [b] = robot_part(R)
% [R] = robot_part(R, block, ...)
% [R] = robot_part(R, block, ..., mode)
%
% syntax for multi-segments robots:
% [b] = robot_part(R, nr)
% [R] = robot_part(R, nr, block, ...)
% [R] = robot_part(R, nr, block, ..., mode)
%
% input:
% R         - handle to structure representing a robot
% nr        - joint number which apperance is defined or return
% block,... - a list of blocks' handles defining the apperance of robot
% mode      - determines the mode of apperance definition,
%             allowed values:
%             'add' - blocks list is added to already existing
%             'new' - blocks list defines the new apperance 
%             default 'new'
%
% output:
% b - a list of blocks' handles defining the apperance of robot
% R - handle to structure representing a robot with modified apperance
%
% examples:
%
% r = robot( [dh(0,0,0,-pi/4); dh(0,7,0,pi/2)], ['r', 'r'],...
%            'RobotBase', rp2t(rotx(-pi/2),[0;0;0]), 'RobotGrasper', [5; 0; -1]);
% b = block_move(cuboid(1, 1, 10), rp2t(roty(pi/2),[-2.5;0;0]));
% r = robot_part(r, 1, b);
% b1 = block_move(cuboid(1, 1, 5), rp2t(roty(pi/2),[-0.5;0;-1]));
% b2 = block_move(cylinder(0.2, 2), [0;0;-1.5]);
% r = robot_part(r, 2, b1, b2);
% b1 = block_move(cuboid(1,1,0.25), [0;0;0.25]);
% b2 = block_move(cuboid(1,1,0.25), [0;0;-0.5]);
% b12 = block_complex(b1,b2)
% r = robot_part(r, 3, b12);
% axis([-5 15 -10 10 -5 10])
% robot_plot(r, 'mesh')
% robot_plot_csys(r)
% campos([2,-1,1.5])
% creates and plots a robot with two revolute joints, each arm is a cuboid
% with base's edges length equal 1, first and second arm are connected by 
% cylinder, grasper is build as a complex block consisting of two small cuboids
%
% r1 = robot( [dh(0,0,0,-pi/2); dh(0,4,0,-pi/2)], ['r', 'r'],...
%             'RobotBase', rp2t(rotx(-pi/2),[2;2;0]),...
%             'RobotGrasper', [2; 0; 0], 'RobotGrasperSize', 1, ...
%             'RobotCylinderRadius', 0.25);
% r2 = robot( [dh(0,0,0,-pi/4); dh(0,4,0,-pi/2)], ['r', 'r'],...
%             'RobotBase', rp2t(rotx(-pi/2),[2;-2;0]),...
%             'RobotGrasper', [2; 0; 0], 'RobotGrasperSize', 1, ...
%             'RobotCylinderRadius', 0.25);
% r = robot_complex(r1, r2);
% b1 = block_move(cube(1), [0;0.5;-0.5]);
% b2 = block_move(cube(1), [0;0.5;-4.5]);
% b3 = block_move(cuboid(3,0.5,7), [0;1.25;-5.5]);
% robot_part(r, b1, b2, b3)
% axis([-5 5 -5 5 -2 10]);
% robot_plot(r, 'surf');
% view([-52,30])
% camlight
% creates a mechanism consisting of two planar manipulators with two
% revolute joints, the connection between the manipulators is defined by 
% user as a platform, with two cube-shaped bases for each manipulator
%
% See also: ROBOT, BLOCK, BLOCK_COMPLEX 
%

global RTHND;

if ( ~parIsObj(R,'robot') ) rethrow(geterr('argValObj', 'R', 'robot')); end

nr = 0;
if ( parIsObj(R,'robot:manip') )
  if ( length(varargin) == 0 ) rethrow(geterr('argNum')); end
  nr = varargin{1};  varargin = varargin(2:length(varargin));
  if ( ~parIsVector(nr,1)   ) rethrow(geterr('argValNum', 'nr')); end
  if (( nr < 0 ) | ( nr > RTHND{R}.n+1 )) 
    rethrow(geterr('argVal', 'nr', sprintf('%d..%d',0,RTHND{R}.n+1))); 
  end
end

if ( length(varargin) > 0 )
    
  if ( ischar(varargin{length(varargin)}) )
    switch lower(varargin{length(varargin)}) 
      case 'new', mode = 1;
      case 'add', mode = 2;
      otherwise, rethrow(geterr('argValObj', 'block', 'block'));
    end;
    varargin = varargin(1:length(varargin)-1);
  else mode=1; end
  
  for i=1:length(varargin) 
    if ( ~parIsObj(varargin{i},'block') ) rethrow(geterr('argValObj', 'block', 'block')); end
  end
  
  for i=1:length(varargin) fhnddel(varargin{i}); blkdel(varargin{i},'all',1); end
  
  if (( mode == 1 ) | ( RTHND{R}.user3d.blocks(nr+1) == 0 ))
    if ( RTHND{R}.user3d.blocks(nr+1) ~= 0 ) 
      blkdel(RTHND{R}.user3d.blocks(nr+1),'all',1);
      RTHND{RTHND{R}.user3d.blocks(nr+1)} = 0;
    end
    RTHND{R}.user3d.blocks(nr+1) = block_complex(varargin{:});
    RTHND{RTHND{R}.user3d.blocks(nr+1)}.owner = R;
  else
    RTHND{R}.user3d.blocks(nr+1) = block_complex(RTHND{R}.user3d.blocks(nr+1), varargin{:});
  end;
  
  axs = getowner(R, 'axes');
  if ( axs ~= -1 )
    hold(axs, 'on');
    R = rrdrw(R, axs, 1);
    hold(axs, 'off');
  end

else

  R = RTHND{R}.user3d.blocks(nr+1); 

end

if ( nargout == 0 ) clear R; end
