
function [R] = robot_plot(R, view)

%ROBOT_PLOT plots the robot in figure window
% 
% syntax:
% [R] = robot_plot(R)
% [R] = robot_plot(R, view)
%
% Parameters of the robot are defined by its properties,
% see function 'robot'.
%
% input:
% R    - handle to structure representing a robot to be plotted
% view - method of plotting, allowed values:
%        'line' - each nonzero length and offset of the robot are plotted
%                 as a line
%        'mesh' - robot is plotted as a wireframe 3D object, if user doesn't 
%                 define 3D view default view is used
%        'surf' - robot is plotted as a set of patches, if user doesn't define
%                 3D view default view is used
%        'dmesh', 'dsurf' - robot is plotted in default 3D view even if
%                           user view is defined
%        'umesh', 'usurf' - robot is plotted in user 3D view, an error occurs
%                           if this view is not defined
%        default value 'line'
%
% output:
% R - handle to structure representing a robot
%
% example:
% axis([-5 15 -10 10 -5 10]);
% r = robot([dh(0,0,0,-pi/4); dh(0,5,0,pi/2); dh(0,5,0,-pi/4)], ['r','r','r'], ...
%           'RobotBase', rp2t(rotx(-pi/2),[0;0;0]), 'RobotModel', 'configex', ...
%           'RobotCylinderRadius', 0.5, 'RobotGrasper', [3; 0; 0], ...
%           'RobotGrasperSize', 1);
% robot_plot(r, 'mesh');
% creates and plots a planar manipulator with three revolute kinematics pairs 
% in default 3D view
%
% See also: ROBOT, ROBOT_PLOT_CSYS, ROBOT_DELETE
%

global RTHND;

if ( ~parIsObj(R,'robot') ) rethrow(geterr('argValObj', 'R', 'robot')); end
if ( RTHND{R}.owner  & parIsObj(RTHND{R}.owner,'robot') ) rethrow(geterr('argValObj0','R','robot')); end

if ( nargin < 2 ) view = 0; end
view = view2num(view, [0,1,2,3,4,5,6]);

if ( RTHND{R}.fig.axes == -1 ) RTHND{R}.fig.axes=getfigaxs(); fhndadd(R); end

hold(RTHND{R}.fig.axes, 'on');
R = rplt(R, eye(4), RTHND{R}.fig.axes, view);
hold(RTHND{R}.fig.axes, 'off');


if ( nargout == 0 ) clear R; end
