
function [R] = robot_plot_csys(R, nr, base, grasper)

%ROBOT_PLOT_CSYS plots the robot's coordinates systems in figure window
% 
% syntax:
% [R] = robot_plot_csys(R)
% [R] = robot_plot_csys(R, nr)
% [R] = robot_plot_csys(R, nr, base)
% [R] = robot_plot_csys(R, nr, base, grasper)
%
% Parameters of the coordinate systems are defined by properties of robot,
% see function 'robot'.
%
% input:
% R       - handle to structure representing a robot to be plotted
% nr      - boolean value or vector, sets visibility individual robot's
%           coordinates systems
% base    - boolean value, sets visibility of base coordinate system
% grasper - boolean value, sets visibility of grasper coordinate system
%
% output:
% R - handle to structure representing a robot
%
% example:
% axis([-5 15 -10 10 -5 10]);
% r = robot([dh(0,0,0,-pi/4); dh(0,5,0,pi/2); dh(0,5,0,-pi/4)], ['r','r','r'], ...
%           'RobotBase', rp2t(rotx(-pi/2),[0;0;0]), 'RobotModel', 'configex', ...
%           'RobotCylinderRadius', 0.5, 'RobotGrasper', [3; 0; 0], ...
%           'RobotGrasperSize', 1, 'CSysAxisLength', 2);
% r = robot_plot(r, 'mesh');
% r = robot_plot_csys(r, [0 1 0], 1, 1);
% creates and plots a planar manipulator with three revolute kinematics pairs 
% in default 3D view and coordinates systems connected to second joint, base 
% and grasper
%
% See also: ROBOT, ROBOT_PLOT, ROBOT_DELETE
%

global RTHND;

if ( ~parIsObj(R,'robot') ) rethrow(geterr('argValObj', 'R', 'robot')); end

owner=getowner(R,'hnd'); axs=RTHND{owner}.fig.axes;
if ( axs == -1 ) axs=getfigaxs(); RTHND{owner}.fig.axes=axs; fhndadd(owner); end

switch ( RTHND{R}.ident ) 
    
  case 'robot:complex'

    hold(axs, 'on');
    R = rpltcsys(R, axs, nr);
    hold(axs, 'off');
    
  case 'robot:manip'
      
    if ( nargin < 4 ) grasper = 0; end
    if ( nargin < 3 ) base = 0; end
    if ( nargin < 2 ) nr = ones(1, RTHND{R}.n); end

    if ( length(nr) == 1 ) nr=ones(1, RTHND{R}.n)*nr; end

    hold(axs, 'on');
    R = rpltcsys(R, axs, [base, nr, grasper]);
    hold(axs, 'off');
    

  case 'robot:platform'

    if ( nargin < 3 ) base = 0; end
    if ( nargin < 2 ) nr   = 1; end

    hold(axs, 'on');
    R = rpltcsys(R, axs, [base, nr]);
    hold(axs, 'off');
    
end

if ( nargout == 0 ) clear R; end
