
function [R] = robot_set(R, varargin)

%ROBOT_SET sets the values of the robot's properties
% 
% syntax:
% [R] = robot_set(R, 'PropertyName', PropertyValue, ...)
% [R] = robot_set(R, nr, 'PropertyName', PropertyValue, ...)
%
% input:
% R  - handle to structure representing a robot
% nr - index of the robot in the complex robot, only if R is a mechanism 
%      defined by robot_complex; index of the robot corresponds to the 
%      structure of the mechanism; see example II
% property name/property value pairs are used to specify set properties,
% see function 'robot'
%
% output:
% R - handle to structure representing a robot
%
% examples:
%
% axis([-5 15 -10 10 -5 10]);
% r = robot([dh(0,0,0,-pi/4); dh(0,5,0,pi/2); dh(0,5,0,-pi/4)], ['r','r','r'], ...
%           'RobotBase', rp2t(rotx(-pi/2),[0;0;0]), 'RobotModel', 'configex', ...
%           'RobotGrasper', [3; 0; 0], 'RobotGrasperSize', 1);
% robot_plot(r, 'mesh');
% pause
% robot_set(r, 'RobotColor', 'k', 'RobotCylinderRadius', 0.5);
% creates and plots a planar manipulator with three revolute kinematics 
% pairs, next after pressig any key sets its color to black and radius 
% of cylinders (default 3D view) to 0.5
%
% axis([-5 10 -7 7 -5 10]);
% r1 = robot([dh(0,0,0,-pi/2); dh(0,4,0,-pi/2)], ['r', 'r'], ...
%            'RobotBase', rp2t(rotx(-pi/2),[2;2;0]), 'RobotColor', 'r', ...
%            'RobotGrasper', [3; 0; 0], 'RobotGrasperSize', 1);
% r2 = robot([dh(0,0,0,-pi/4); dh(0,4,0,-pi/2)], ['r', 'r'], ...
%            'RobotBase', rp2t(rotx(-pi/2),[2;-2;0]), 'RobotColor', 'b', ...
%            'RobotGrasper', [3; 0; 0], 'RobotGrasperSize', 1);
% r  = robot_complex(r1, r2, 'RobotColor', 'g');
% robot_plot(r)
% pause
% robot_set(r, 'RobotColor', 'k')
% robot_set(r, 2, 'RobotColor', 'm')
% creates and plots a mechanism consisting of two planar manipulators with 
% two revolute joints, next sets color of the complex robot (connection 
% between r1 and r2) to black and sets the color of the second manipulator 
% (manipulator r2 which has been added as a second item) to magenta
%
% See also: ROBOT, ROBOT_GET, ROBOT_INFO
%

global RTHND;

if ( ~parIsObj(R,'robot') ) rethrow(geterr('argValObj', 'R', 'robot')); end

nr = 0;   r = R;
if ( isnumeric(varargin{1}) )
  if ( ~parIsObj(R,'robot:complex') ) rethrow(geterr('argValObj', 'R', 'robot:complex')); end
  nr = varargin{1};  
  if (( nr < 1 ) | ( nr > length(RTHND{R}.robots) )) rethrow(geterr('argValObj1', 'nr')); end
  r = RTHND{R}.robots(nr);
  varargin = varargin(2:length(varargin));
end

try   [lst, setlst] = prepopts(RTHND{r}.ident, varargin);
catch rethrow(lasterror); end;

for i=1:size(setlst,1) 
  RTHND{r} = rset(RTHND{r}, setlst{i,1}, setlst{i,2}, setlst{i,4}, 0); 
end

axs = getowner(R, 'axes'); 
if ( axs ~= -1 )
  hold(axs, 'on');
  r = rrdrw(R, axs, 1);
  hold(axs, 'off');
end

if ( nargout == 0 ) clear R; end
