
function [q, v, a, t, c] = trj_line(t, q0, qt, a0, at, v0, vt)

%TRAJECTORY generates the robot's trajectory described by linear segment
%           with parabolic blends from initial configuration q0 to final 
%           configuration qt
% 
% Trajectory of each joint is described by dependencies
%
%        for t0<=t<t1:  a2*t^2 + a1*t + a0
% q(t) = for t1<=t<t2:  b1*t + b0
%        for t2<=t<tt:  c2*t^2 + c1*t + c0
%
% syntax:
% [q, v, a, t, c] = trj_line(t, q0, qt, a0, at)
% [q, v, a, t, c] = trj_line(t, q0, qt, a0, at, v0, vt)
%
% input:
% t      - number or vector, if t is a number function calculates
%          trajectory from 0 to t, if t is a vector trajectory is 
%          calculated for each time instant from t
% q0, qt - vector, initial and final configuration of robot
% a0, at - number or vector, initial and final acceleration, number means
%          the same value for all joints
% v0, vt - number or vector, initial and final velocity, number means
%          the same value for all joints, default 0
%
% output:
% q - row matrix, i-th row containts a trajectory for i-th joint
% v - row matrix, i-th row containts the instantaneous velocities of i-th joint
% a - row matrix, i-th row containts the instantaneous accelerations of i-th joint
% t - row vector of time points
% c - column matrix, i-th column containts polynomials' coefficients
%     describing trajectory for i-th joint, in some cases containts also
%     additional parameters
%
% example:
% [q, v, a, t] = trj_line(5:0.025:10, [0; 2], [3; -1], 1, 1);
% figure(1); plot(t,q(1,:), t,v(1,:), t,a(1,:))
% figure(2); plot(t,q(2,:), t,v(2,:), t,a(2,:))
% generates linear segment with parabolic blends trajectory for manipulator 
% with two joints, manipulator starts from the configuration q0=[0;2] and 
% ends in the configuration qt=[3;-1], the accelerations for parabolic
% segments equals 1, the start and final velocities are default(0); 
% calculated trajectory, velocities and accelerations for each joint are 
% plotted in separate figure
%
% See also: TRAJECTORY, TRJ_POLY3, TRJ_POLY5
%

if ( length(t) == 1 ) 
  t0=0; tt=t; t=(t0:0.1:tt);
else
  t0=t(1); tt=t(length(t));
  if ( size(t,2) == 1 ) t=t'; end
end

if ( ~parIsMatrix(q0,0,1) )     rethrow(geterr('argValCVect', 'q0')); end
if ( ~parIsMatrix(qt,0,1) )     rethrow(geterr('argValCVect', 'qt')); end
if ( length(q0) ~= length(qt) ) rethrow(geterr('trjQLen')); end

n = length(q0);

if ( nargin < 7 ) vt=zeros(n,1); end
if ( nargin < 6 ) v0=zeros(n,1); end
if ( length(v0) == 1 ) v0=v0*ones(n,1); end
if ( length(vt) == 1 ) vt=vt*ones(n,1); end
if (( length(v0) ~= n ) | ( length(vt) ~= n )) rethrow(geterr('trjVLen')); end

if ( nargin < 5 ) rethrow(geterr('trjAUndef')); end
if ( nargin < 4 ) rethrow(geterr('trjAUndef')); end
if ( length(a0) == 1 ) a0=a0*ones(n,1); end
if ( length(at) == 1 ) at=at*ones(n,1); end
if (( length(a0) ~= n ) | ( length(at) ~= n )) rethrow(geterr('trjVLen')); end
if ( any(a0==0) | any(at==0) ) rethrow(geterr('trjAZero')); end

c = trjline(t0,tt,q0,qt,v0,vt,a0,at);
    
q = zeros(n,length(t));  
v = zeros(n,length(t));
a = zeros(n,length(t));
for i=1:n
  if ( nargout == 0 ) disp(sprintf('\n%2d. polaczenie:\n---------------\n', i)); end
  if ( all(c(:,i)==0) )
    warning('ROBOT:trjLineA', getwrn('trjLineA'));
  else
    if ( nargout == 0 )
      disp(sprintf('        %.2f<=t<%.2f:  %.2ft^2 + %.2ft + %.2f', t(1), c(9,i), c(1,i), c(2,i), c(3,i)));
      disp(sprintf(' q(t) = %.2f<=t<%.2f:  %.2ft + %.2f', c(9,i), c(10,i), c(4,i), c(5,i)));
      disp(sprintf('        %.2f<=t<%.2f:  %.2ft^2 + %.2ft + %.2f\n', c(10,i), t(length(t)), c(6,i), c(7,i), c(8,i)));
    end
    t1 = t(1:find(t<=c(9,i),1,'last'));
    t2 = t(length(t1)+1:find(t<=c(10,i),1,'last'));
    t3 = t(length(t1)+length(t2)+1:length(t));
    q(i,:) = [polyval(c(1:3,i),t1) polyval(c(4:5,i),t2) polyval(c(6:8,i),t3)];
    v(i,:) = [polyval([2;1].*c(1:2,i),t1) c(4,i)*ones(1,length(t2)) polyval([2;1].*c(6:7,i),t3)];
    a(i,:) = [2*c(1,i)*ones(1,length(t1)) zeros(1,length(t2)) 2*c(6,i)*ones(1,length(t3))];
  end
end
