
function [q, v, a, t, c] = trj_poly5(t, q0, qt, a0, at, v0, vt)

%TRJ_POLY5 generates the robot's trajectory described by 5th order polynomial
%          from initial configuration q0 to final configuration qt
% 
% Trajectory of each joint is described by dependency
%
% q(t) = a5*t^5 + a4*t^4 + a3*t^3 + a2*t^2 + a1*t + a0
%
% syntax:
% [q, v, a, t, c] = trj_poly5(t, q0, qt)
% [q, v, a, t, c] = trj_poly5(t, q0, qt, a0, at)
% [q, v, a, t, c] = trj_poly5(t, q0, qt, a0, at, v0, vt)
%
% input:
% t      - number or vector, if t is a number function calculates
%          trajectory from 0 to t, if t is a vector trajectory is 
%          calculated for each time instant from t
% q0, qt - vector, initial and final configuration of robot
% a0, at - number or vector, initial and final acceleration, number means
%          the same value for all joints, default 0
% v0, vt - number or vector, initial and final velocity, number means
%          the same value for all joints, default 0
%
% output:
% q - row matrix, i-th row containts a trajectory for i-th joint
% v - row matrix, i-th row containts the instantaneous velocities of i-th joint
% a - row matrix, i-th row containts the instantaneous accelerations of i-th joint
% t - row vector of time points
% c - column matrix, i-th column containts polynomials' coefficients
%     describing trajectory for i-th joint, c=[a5; a4; a3; a2; a1; a0]
%
% example:
% [q, v, a, t] = trj_poly5(5:0.025:10, [0; 2], [3; -1], 1, 1);
% figure(1); plot(t,q(1,:), t,v(1,:), t,a(1,:))
% figure(2); plot(t,q(2,:), t,v(2,:), t,a(2,:))
% generates polynomials trajectory for manipulator with two joints, 
% manipulator starts from the configuration q0=[0;2] and ends in the 
% configuration qt=[3;-1], the start and final accelerations equals 1,
% velocities are default (0), calculated trajectory, velocities and 
% accelerations for each joint are plotted in separate figure
%
% See also: TRAJECTORY, TRJ_POLY3, TRJ_LINE
%

if ( length(t) == 1 ) 
  t0=0; tt=t; t=(t0:0.1:tt);
else
  t0=t(1); tt=t(length(t));
  if ( size(t,2) == 1 ) t=t'; end
end

if ( ~parIsMatrix(q0,0,1) )     rethrow(geterr('argValCVect', 'q0')); end
if ( ~parIsMatrix(qt,0,1) )     rethrow(geterr('argValCVect', 'qt')); end
if ( length(q0) ~= length(qt) ) rethrow(geterr('trjQLen')); end

n = length(q0);

if ( nargin < 7 ) vt=zeros(n,1); end
if ( nargin < 6 ) v0=zeros(n,1); end
if ( length(v0) == 1 ) v0=v0*ones(n,1); end
if ( length(vt) == 1 ) vt=vt*ones(n,1); end
if (( length(v0) ~= n ) | ( length(vt) ~= n )) rethrow(geterr('trjVLen')); end

if ( nargin < 5 ) at=zeros(n,1); end
if ( nargin < 4 ) a0=zeros(n,1); end
if ( length(a0) == 1 ) a0=a0*ones(n,1); end
if ( length(at) == 1 ) at=at*ones(n,1); end
if (( length(a0) ~= n ) | ( length(at) ~= n )) rethrow(geterr('trjVLen')); end
  
c = trjpoly5(t0,tt,q0,qt,v0,vt,a0,at);
    
q = zeros(n, length(t));
v = zeros(n, length(t));
a = zeros(n, length(t));
for i=1:n
  if ( nargout == 0 )
    disp(sprintf('\n%2d. polaczenie:\n---------------\n', i));
    disp(sprintf(' a5=%.5f, a4=%.5f, a3=%.5f, a2=%.5f, a1=%.5f, a0=%.5f\n', c(1,i), c(2,i), c(3,i), c(4,i), c(5,i), c(6,i)));
    disp(sprintf(' q(t) = %.2ft^5 + %.2ft^4 + %.2ft^3 + %.2ft^2 + %.2ft + %.2f\n', c(1,i), c(2,i), c(3,i), c(4,i), c(5,i), c(6,i)));
  end
  q(i,:) = polyval(c(:,i),t);
  v(i,:) = polyval([5;4;3;2;1].*c(1:5,i),t);
  a(i,:) = polyval([20;12;6;2].*c(1:4,i),t);
end
